@@ -2,8 +2,11 @@ from __future__ import print_function
 import argparse
 import autopep8
 from itertools import takewhile
+import glob
 import re
-from subprocess import check_output, STDOUT, CalledProcessError
+import signal
+import subprocess
+from subprocess import STDOUT, CalledProcessError
 import sys
 from sys import exit
 
@@ -12,8 +15,28 @@ try:
 except ImportError:
     from io import StringIO
 
+# python 2.6 doesn't include check_output
+# http://hg.python.org/cpython/file/d37f963394aa/Lib/subprocess.py#l544
+if "check_output" not in dir(subprocess):  # duck punch it in!
+    def f(*popenargs, **kwargs):
+        if 'stdout' in kwargs:
+            raise ValueError(
+                'stdout argument not allowed, it will be overridden.')
+        process = subprocess.Popen(
+            stdout=subprocess.PIPE, *popenargs, **kwargs)
+        output, unused_err = process.communicate()
+        retcode = process.poll()
+        if retcode:
+            cmd = kwargs.get("args")
+            if cmd is None:
+                cmd = popenargs[0]
+            raise subprocess.CalledProcessError(retcode, cmd)
+        return output
+    subprocess.check_output = f
+check_output = subprocess.check_output
 
-__version__ = version = '0.5'
+
+__version__ = version = '0.6'
 
 
 DEFAULT_IGNORE = 'E24'
@@ -21,10 +44,51 @@ DEFAULT_INDENT_SIZE = 4
 
 
 def main():
-    description = ("Tidy up (autopep8) only the lines in the files touched "
-                   "in the git branch/commit.")
-    epilog = ("Run before you do a commit to tidy, "
-              "or against a previous commit or branch before merging.")
+    """Tool main."""
+    try:
+        # Exit on broken pipe.
+        signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+    except AttributeError:  # pragma: no cover
+        # SIGPIPE is not available on Windows.
+        pass
+
+    try:
+        args = parse_args(sys.argv[1:])
+
+        # main
+        if args.version:
+            print(version)
+            exit(0)
+
+        if args.list_fixes:
+            for code, description in sorted(autopep8.supported_fixes()):
+                print('{code} - {description}'.format(
+                    code=code, description=description))
+            exit(0)
+
+        try:
+            r = Radius.new(rev=args.rev, options=args)
+        except NotImplementedError as e:
+            print(e.message)
+            exit(1)
+        except CalledProcessError as c:
+            # cut off usage of git diff and exit
+            output = c.output.splitlines()[0]
+            print(output)
+            exit(c.returncode)
+
+        r.pep8radius()
+
+    except KeyboardInterrupt:
+        return 1  # pragma: no cover
+
+
+def parse_args(arguments):
+    description = ("PEP8 clean only the parts of the files which you have "
+                   "touched since the last commit, previous commit or "
+                   "branch.")
+    epilog = ("Run before you commit, or against a previous commit or "
+              "branch before merging.")
     parser = argparse.ArgumentParser(description=description,
                                      epilog=epilog)
     parser.add_argument('rev',
@@ -71,9 +135,10 @@ def main():
                         type=int, metavar='n',
                         help='number of spaces per indent level '
                              '(default %(default)s)')
-    args = parser.parse_args()
 
-    # sanity check args (fro autopep8)
+    args = parser.parse_args(arguments)
+
+    # sanity check args (from autopep8)
     if args.max_line_length <= 0:
         parser.error('--max-line-length must be greater than 0')
 
@@ -89,35 +154,14 @@ def main():
         else:
             args.ignore = DEFAULT_IGNORE.split(',')
 
-    if args.exclude:
-        args.exclude = args.exclude.split(',')
-    else:
-        args.exclude = []
-
-    # main
-    if args.version:
-        print(version)
-        exit(0)
-
-    try:
-        r = Radius.new(rev=args.rev, options=args)
-    except NotImplementedError as e:
-        print(e.message)
-        exit(1)
-    except CalledProcessError as c:
-        # cut off usage of git diff and exit
-        output = c.output.splitlines()[0]
-        print(output)
-        exit(c.returncode)
-
-    r.pep8radius()
+    return args
 
 
 class Radius:
 
     def __init__(self, rev=None, options=None):
         self.rev = rev if rev is not None else self.current_branch()
-        self.options = options if options else autopep8.parse_args([''])
+        self.options = options if options else parse_args([''])
         self.verbose = self.options.verbose
         self.dry_run = self.options.dry_run
         self.diff = self.options.diff or self.options.dry_run
@@ -195,7 +239,6 @@ class Radius:
         partial = original
         for start, end in self.line_numbers_from_file_diff(diff):
             partial = self.autopep8_line_range(partial, start, end)
-            # import pdb; pdb.set_trace()
             self.p('.', end='', max_=1)
         self.p('', max_=1)
         fixed = partial
@@ -222,11 +265,17 @@ class Radius:
         diff_files_u = diff_files_b.decode('utf-8')
         diff_files = self.parse_diff_filenames(diff_files_u)
 
-        return [f for f in diff_files if f.endswith('.py')]
+        py_files = set(f for f in diff_files if f.endswith('.py'))
+
+        if self.options.exclude:
+            excludes = glob.glob(self.options.exclude)
+            py_files.difference_update(excludes)
+        return list(py_files)
 
     def line_numbers_from_file_diff(self, diff):
         "Potentially this is vc specific (if not using udiff)"
-        return line_numbers_from_file_udiff(diff)
+        for start, end in line_numbers_from_file_udiff(diff):
+            yield start, end
 
     def p(self, something_to_print, end=None, min_=1, max_=99):
         if min_ <= self.verbose <= max_:
@@ -244,17 +293,22 @@ def line_numbers_from_file_udiff(udiff):
     Note: returned in descending order (as autopep8 can +- lines)
 
     """
-    chunks = re.split('\n@@[^\n]+\n', udiff)[1:]
+    chunks = re.split('\n@@[^\n]+\n', udiff)[:0:-1]
 
     line_numbers = re.findall('(?<=[+])\d+(?=,\d+)', udiff)
-    line_numbers = map(int, line_numbers)
+    line_numbers = list(map(int, line_numbers))[::-1]
 
-    # Note: this is reversed as can modify number of lines
-    for c, start in reversed(zip(chunks, line_numbers)):
+    # Note: these were reversed as can modify number of lines
+    for c, start in zip(chunks, line_numbers):
         empty = [line.startswith(' ') for line in c.splitlines()]
         pre_padding = sum(1 for _ in takewhile(lambda b: b, empty))
-        new_lines = sum(line.startswith('+') for line in c)
-        yield (start + pre_padding, start + pre_padding + new_lines)
+        post_padding = sum(1 for _ in takewhile(lambda b: b, empty[::-1]))
+
+        sub_lines = sum(line.startswith('-') for line in c.splitlines())
+        lines_in_range = len(c.splitlines()) - sub_lines - post_padding
+
+        yield (start + pre_padding,
+               start + lines_in_range - 1)
 
 
 def udiff_lines_changes(u):
@@ -297,7 +351,7 @@ class RadiusHg(Radius):
 
     def file_diff_cmd(self, f):
         "Get diff for one file, f"
-        return ['hg', 'diff', '-c', self.rev, f]
+        return ['hg', 'diff', '-r', self.rev, f]
 
     def filenames_diff_cmd(self):
         "Get the names of the py files in diff"
